<?php
	/*!	@class		SocketServer
		@author		Navarr Barnier
		@abstract 	A Framework for creating a multi-client server using the PHP language.
	 */
	class SocketServer
	{
	    const SYSTEM_ENVIRONMENT = "PROD";//"DEV";
	    
		/*!	@var		config
			@abstract	Array - an array of configuration information used by the server.
		 */
		protected $config;

		/*!	@var		hooks
			@abstract	Array - a dictionary of hooks and the callbacks attached to them.
		 */
		protected $hooks;

		/*!	@var		master_socket
			@abstract	resource - The master socket used by the server.
		 */
		protected $master_socket;
		
		/*!	@var		max_clients
			@abstract	unsigned int - The maximum number of clients allowed to connect.
		 */
		public $max_clients = 10;

		/*!	@var		max_read
			@abstract	unsigned int - The maximum number of bytes to read from a socket at a single time.
		 */
		public $max_read = 1024;

		/*!	@var		clients
			@abstract	Array - an array of connected clients.
		 */
		public $clients;

		/*!	@function	__construct
			@abstract	Creates the socket and starts listening to it.
			@param		string	- IP Address to bind to, NULL for default.
			@param		int	- Port to bind to
			@result		void
		 */
		public function __construct($bind_ip,$port)
		{
			set_time_limit(0);
			$this->hooks = array();

			$this->config["ip"] = $bind_ip;
			$this->config["port"] = $port;

			$this->master_socket = socket_create(AF_INET, SOCK_STREAM, 0);
			socket_bind($this->master_socket,$this->config["ip"],$this->config["port"]) or die("Issue Binding");
			socket_getsockname($this->master_socket,$bind_ip,$port);
			socket_listen($this->master_socket);
			SocketServer::debug("Listenting for connections on {$bind_ip}:{$port}");
			Logger::getInstance()->lwrite("====================================================");
			Logger::getInstance()->lwrite("DEBUG starting 8BC controller command server");
		}

		/*!	@function	hook
			@abstract	Adds a function to be called whenever a certain action happens.  Can be extended in your implementation.
			@param		string	- Command
			@param		callback- Function to Call.
			@see		unhook
			@see		trigger_hooks
			@result		void
		 */
		public function hook($command,$function)
		{
			$command = strtoupper($command);
			if(!isset($this->hooks[$command])) { $this->hooks[$command] = array(); }
			$k = array_search($function,$this->hooks[$command]);
			if($k === FALSE)
			{
				$this->hooks[$command][] = $function;
			}
		}

		/*!	@function	unhook
			@abstract	Deletes a function from the call list for a certain action.  Can be extended in your implementation.
			@param		string	- Command
			@param		callback- Function to Delete from Call List
			@see		hook
			@see		trigger_hooks
			@result		void
		 */
		public function unhook($command = NULL,$function)
		{
			$command = strtoupper($command);
			if($command !== NULL)
			{
				$k = array_search($function,$this->hooks[$command]);
				if($k !== FALSE)
				{
					unset($this->hooks[$command][$k]);
				}
			} else {
				$k = array_search($this->user_funcs,$function);
				if($k !== FALSE)
				{
					unset($this->user_funcs[$k]);
				}
			}
		}

		/*!	@function	loop_once
			@abstract	Runs the class's actions once.
			@discussion	Should only be used if you want to run additional checks during server operation.  Otherwise, use infinite_loop()
			@param		void
			@see		infinite_loop
			@result 	bool	- True
		*/
		public function loop_once()
		{
			// Setup Clients Listen Socket For Reading
			$read[0] = $this->master_socket;
			$write  = NULL;
			$except = NULL;
			for($i = 0; $i < $this->max_clients; $i++)
			{
				if(isset($this->clients[$i]))
				{
					$read[$i + 1] = $this->clients[$i]->socket;
				}
			}

			// Set up a blocking call to socket_select
			if(socket_select($read,$write, $except, $tv_sec = 5) < 1)
			{
			//	SocketServer::debug("Problem blocking socket_select?");
				return true;
			}

			// Handle new Connections
			if(in_array($this->master_socket, $read))
			{
				for($i = 0; $i < $this->max_clients; $i++)
				{
					if(empty($this->clients[$i]))
					{
						$temp_sock = $this->master_socket;
						$this->clients[$i] = new SocketServerClient($this->master_socket,$i);
						$this->trigger_hooks("CONNECT",$this->clients[$i],"");
						//$read[$i + 1] = $this->clients[$i]->socket;
						break;
					}
					elseif($i == ($this->max_clients-1))
					{
						SocketServer::debug("Too many clients... :( ");
						Logger::getInstance()->lwrite("Too many clients... :");
					}
				}
			}
			// Handle Input
			/*for($i = 0; $i < $this->max_clients; $i++) // for each client
			{
				if(isset($this->clients[$i]))
				{
					if(in_array($this->clients[$i]->socket, $read))
					{
					    $input = socket_read($this->clients[$i]->socket, $this->max_read);
						if(empty($input))
						{
							$this->disconnect($i);
						}
						else
						{
							SocketServer::debug("{$i}@{$this->clients[$i]->ip} --> {$input}");
							$this->trigger_hooks("INPUT",$this->clients[$i],$input);
						}
					}
				}
			}*/
			return true;
		}

		/*!	@function	disconnect
			@abstract	Disconnects a client from the server.
			@param		int	- Index of the client to disconnect.
			@param		string	- Message to send to the hooks
			@result		void
		*/
		public function disconnect($client_index,$message = "")
		{
			$i = $client_index;
			SocketServer::debug("Client {$i} from {$this->clients[$i]->ip} Disconnecting");
			Logger::getInstance()->lwrite("DEBUG Client {$i} from {$this->clients[$i]->ip} Disconnecting");
			$this->trigger_hooks("DISCONNECT",$this->clients[$i],$message);
			$this->clients[$i]->destroy();
			unset($this->clients[$i]);			
		}

		/*!	@function	trigger_hooks
			@abstract	Triggers Hooks for a certain command.
			@param		string	- Command who's hooks you want to trigger.
			@param		object	- The client who activated this command.
			@param		string	- The input from the client, or a message to be sent to the hooks.
			@result		void
		*/
		public function trigger_hooks($command,&$client,$input)
		{
			if(isset($this->hooks[$command]))
			{
				foreach($this->hooks[$command] as $function)
				{
					SocketServer::debug("Triggering Hook '{$function}' for '{$command}'");
					//$continue = call_user_func($function,&$this,&$client,$input);
					$continue = call_user_func_array($function, array(&$this,&$client,$input));
					if($continue === FALSE) { break; }
				}
			}
		}

		/*!	@function	infinite_loop
			@abstract	Runs the server code until the server is shut down.
			@see		loop_once
			@param		void
			@result		void
		*/
		public function infinite_loop()
		{
			$test = true;
			do
			{
				$test = $this->loop_once();
			}
			while($test);
		}

		/*!	@function	debug
			@static
			@abstract	Outputs Text directly.
			@discussion	Yeah, should probably make a way to turn this off.
			@param		string	- Text to Output
			@result		void
		*/
		public static function debug($text)
		{
			echo("{$text}\r\n");
		}

		/*!	@function	socket_write_smart
			@static
			@abstract	Writes data to the socket, including the length of the data, and ends it with a CRLF unless specified.
			@discussion	It is perfectly valid for socket_write_smart to return zero which means no bytes have been written. Be sure to use the === operator to check for FALSE in case of an error. 
			@param		resource- Socket Instance
			@param		string	- Data to write to the socket.
			@param		string	- Data to end the line with.  Specify a "" if you don't want a line end sent.
			@result		mixed	- Returns the number of bytes successfully written to the socket or FALSE on failure. The error code can be retrieved with socket_last_error(). This code may be passed to socket_strerror() to get a textual explanation of the error.
		*/
		public static function socket_write_smart(&$sock,$string,$crlf = "\r\n")
		{
			SocketServer::debug("<-- {$string}");
			if($crlf) { $string = "{$string}{$crlf}"; }
			return socket_write($sock,$string,strlen($string));
		}

		/*!	@function	__get
			@abstract	Magic Method used for allowing the reading of protected variables.
			@discussion	You never need to use this method, simply calling $server->variable works because of this method's existence.
			@param		string	- Variable to retrieve
			@result		mixed	- Returns the reference to the variable called.
		*/
		function &__get($name)
		{
			return $this->{$name};
		}
		
		
		public static function getIp()
		{
		    $ip = shell_exec("ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'");
		    if (empty($ip)) // Check wirless connection
		        $ip = shell_exec("ifconfig wlan0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'");
		    $ip = trim($ip);
		    return $ip;
		}
	}
	
	/*!	@class		SocketServerClient
	 @author		Navarr Barnier
	 @abstract	A Client Instance for use with SocketServer
	 */
	class SocketServerClient
	{
	    const COMMAND_SYNTAX_DELIMETER = ";";
	    const AUTH_SYNTAX_DELIMETER    = "&";
	    const ACK_NO_ERROR             = "no_error";
	    const ACK_ERROR                = "error";
	    
	    /*!	@var 		commands
	     @abstract	array - array of commands to be send to 811
	     */
	    public $commands;
	    /*!	@var		socket
	     @abstract	resource - The client's socket resource, for sending and receiving data with.
	     */
	    protected $socket;
	    
	    /*!	@var		ip
	     @abstract	string - The client's IP address, as seen by the server.
	     */
	    protected $ip;
	    
	    /*!	@var		hostname
	     @abstract	string - The client's hostname, as seen by the server.
	     @discussion	This variable is only set after calling lookup_hostname, as hostname lookups can take up a decent amount of time.
	     @see		lookup_hostname
	     */
	    protected $hostname;
	    
	    /*!	@var		server_clients_index
	     @abstract	int - The index of this client in the SocketServer's client array.
	     */
	    protected $server_clients_index;

	    /*!	@var		server client error array
	     @abstract	array - array of message error.
	     */
	    protected $arrError;
	    
	    /*!	@function	__construct
	     @param		resource- The resource of the socket the client is connecting by, generally the master socket.
	     @param		int	- The Index in the Server's client array.
	     @result		void
	     */
	    public function __construct(&$socket,$i)
	    {
	        $this->server_clients_index = $i;
	        $this->socket = socket_accept($socket) or die("Failed to Accept");
	        SocketServer::debug("New Client Connected");
	        Logger::getInstance()->lwrite("New Client Connected");
	        socket_getpeername($this->socket,$ip);
	        $this->ip = $ip;
	        $this->arrError = array();
	    }
	    
	    /*!	@function	lookup_hostname
	     @abstract	Searches for the user's hostname and stores the result to hostname.
	     @see		hostname
	     @param		void
	     @result		string	- The hostname on success or the IP address on failure.
	     */
	    public function lookup_hostname()
	    {
	        $this->hostname = gethostbyaddr($this->ip);
	        return $this->hostname;
	    }
	    
	    /*!	@function	destroy
	     @abstract	Closes the socket.  Thats pretty much it.
	     @param		void
	     @result		void
	     */
	    public function destroy()
	    {
	        socket_close($this->socket);
	    }
	    
	    function &__get($name)
	    {
	        return $this->{$name};
	    }

	    public function set_commands($input)
	    {
	        $arr = explode(self::COMMAND_SYNTAX_DELIMETER, trim($input));
	        
	        if (!empty($arr))
	        {
	            $this->commands = array_map('trim', $arr);
	        }
	        else
	        {
	            $this->set_error("ERROR: no command set");
	        }
	    }
	    
	    private function set_error($message)
	    {
	        SocketServer::debug($message);
	        Logger::getInstance()->lwrite($message);
	        array_push($this->arrError, $message);
	    }
	    
	    private function has_errors()
	    {
	        if (empty($this->get_errors()))
	        {
	            return false;
	        }
	        return true;
	    }
	    
	    public function get_errors()
	    {
	        return $this->arrError;
	    }
	    
	    private function get_productId()
	    {
	        /* Attempt MySQL server connection. Assuming you are running MySQL
	        server with default setting (user 'root' with no password) */

	        $link = mysqli_connect("localhost", "root", "", "muxlabnetworkcontroller_dev_db");
	        // Check connection
	        if($link === false){
	            $this->set_error("ERROR: Could not connect DB muxlabnetworkcontroller_dev_db");
	            return false;
	        }
	        
	        // Attempt select query execution
	        $sql = "select globalproductlistsavedID from globalproductlistsaved 
            join productglobalattributs_p6 on GlobalProductListSaved_globalproductlistsavedID = globalproductlistsavedID
            join productioattributslist_p6 on ProductGlobalAttributs_P6_productglobalattributs_p6ID = productglobalattributs_p6ID
            where ProductIoAttrIp = '". $this->ip ."'";
	        
	        if($result = mysqli_query($link, $sql)){
	            
	            if(mysqli_num_rows($result) > 0){ //should be just one
	                while($row = mysqli_fetch_array($result)){
	                    return $row["globalproductlistsavedID"];
	                }
	                
	                // Free result set
	                mysqli_free_result($result);
	                
	            } else{
	                SocketServer::debug("No records found.");
	                Logger::getInstance()->lwrite("No records found.");
	            }
	            
	        } else{
	            $this->set_error("ERROR: Could not able to execute $sql. " . mysqli_error($link));
	        }
	        // Close connection
	        mysqli_close($link);
	        return false;
	    }
	    
	    public function send_commands()
	    {
	        if (isset($this->commands))
	        {
	            $ip        = SocketServer::getIp();
	            $url       = 'http://'.$ip.'/mnc/session_api.php';
	            $productId = $this->get_productId();
	            
	            if (SocketServer::SYSTEM_ENVIRONMENT == "DEV")
	            {
	               $url = 'http://816ip/mnc/session_api.php';
	            }
	   
	            if ($productId)
	            {
    	            foreach ($this->commands as $command)
    	            {
    	                $data_string  = '{"p_targetId":'. $productId .', "p_cmd":"run_command", ';
    	                $data_string .= '"p_data":[{"ip":"'. $this->ip. '", "commandKey":"'. $command . '"}]}';
    	                
    	                $ch=curl_init($url);
    	                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
    	                curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
    	                curl_setopt($ch, CURLOPT_HEADER, true);
    	                curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json', 'Content-Length: ' . strlen($data_string)));
    	                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);//Return response instead of printing.
    	                
    	                $result = curl_exec($ch);
    	                SocketServer::debug(print_r($result, true));
    	                Logger::getInstance()->lwrite("8BC command server CURL response: \n".print_r($result, true));
    	                if (strpos($result, "FAILED") !== false)
    	                {
    	                    $this->set_error("ERROR: 8BC command server CURL response failed");
    	                }
    	                curl_close($ch);
    	            }
	            }
	        }
	    }
	    
	    public function send_ack()
	    {
	        /* Create a TCP/IP socket. */
	        $socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
	        
	        if ($socket === false) {
	            SocketServer::debug("socket_create() -ack- failed: reason: " . socket_strerror(socket_last_error()));
	            Logger::getInstance()->lwrite("socket_create() -ack- failed: reason: " . socket_strerror(socket_last_error()));
	        }
	        
	        $result = socket_connect($socket, $this->ip, 31337);
	        if ($result === false) {
	            SocketServer::debug("socket_connect() -ack- failed.\nReason: ($result) " . socket_strerror(socket_last_error($socket)));
	            Logger::getInstance()->lwrite("socket_connect() -ack- failed.\nReason: ($result) " . socket_strerror(socket_last_error($socket)));
	        }
	        
	        $ack_return = self::ACK_ERROR;
	        if (empty($this->get_errors()))
	        {
	            $ack_return = self::ACK_NO_ERROR;
	        }
	        
	        SocketServer::debug("sending ack ".$ack_return);
	        Logger::getInstance()->lwrite("sending ack ".$ack_return);
	        socket_write($socket, $ack_return, strlen($ack_return));
	        socket_close($socket);
	    }
	    
	    function __isset($name)
	    {
	        return isset($this->{$name});
	    }
	}
	
	/* Logger class:
	 * - contains lopen and lwrite methods
	 * - lwrite will write message to the log file
	 * - first call of the lwrite will open log file implicitly
	 * - message is written with the following format: hh:mm:ss (script name) message
	 */
	class Logger{
	    
	    // define log file
	    private $log_file = 'NetworkController_logfile';
	    // define file pointer
	    
	    private $fp = null;
	    // write message to the log file
	    
	    private $enabled = null;
	    
	    private static $instance;
	    
	    public function lwrite($message){
	        if ($this->enabled == TRUE){
	            // if file pointer doesn't exist, then open log file
	            if (!$this->fp) $this->lopen();
	            // define current time
	            $time = date('H:i:s');
	            // write current time, script name and message to the log file
	            fwrite($this->fp, "$time (command_server) $message\n");
	            //close file
	            //fclose($this->fp);
	        }
	    }
	    
	    
	    private function __construct(){
	        $this->enabled = TRUE;
	    }
	    
	    public function enableLogger() {
	        $this->enabled = TRUE;
	    }
	    
	    public function disableLogger() {
	        $this->enabled = FALSE;
	    }
	    
	    public static function getInstance() {
	        if (!isset(self::$instance)) {
	            //$c = __CLASS__;
	            //self::$instance = new $c;
	            self::$instance = new Logger();
	        }
	        return self::$instance;
	    }
	    
	    // open log file
	    private function lopen(){
	        //log file path
	        if (SocketServer::SYSTEM_ENVIRONMENT == "DEV")
	        {
	            $path = "/home/simith/workspace/ControllerMNC/trunk/Software/For_Web/";
	        }
	        else
	        {
	            $path = "/var/www/html/mnc/";
	        }
	        
	        // define log file path and name
	        $lfile = $path.$this->log_file;
	        // define the current date (it will be appended to the log file name)
	        $today = date('Y-m-d');
	        // open log file for writing only; place the file pointer at the end of the file
	        // if the file does not exist, attempt to create it
	        //$this->fp = fopen($lfile . '_' . $today, 'a') or exit("Can't open $lfile!");
	        $this->fp = fopen($lfile . '_' . $today .'.log', 'a') or exit("Can't open $lfile!");
			$res=shell_exec("chmod -f 764 /var/www/html/mnc/*.log");    
	    }
	}
	
	
